/*****************************************************************************\
 *                                                                           *
 * Configuration.h                                                           *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
 * Defines an object which reads in and parses program variables and         *
 * configuration options for xrttigen.                                       *
 *                                                                           *
\*****************************************************************************/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <Essential/File.h>
#include <Essential/InputStream.h>
#include <Essential/OutputStream.h>
#include <Essential/RefCounted.h>
#include <Essential/RefVector.h>
#include <Essential/String.h>
#include <Essential/Vector.h>

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


class Configuration
{
public:

    class Clude : public RefCounted
    {
    public:

        enum Type
        {
            TypeIn, TypeEx
        };

        enum Match
        {
            MatchExact, MatchSubOf, MatchHas
        };

        Clude(Type type, char *name);

        bool Matches(const String &name);

        Type type;
        Match match;
        String name;
        bool wildcardStartM, wildcardEndM;
    };

    Configuration(int argc, char **argv);

    U32 GetDefinitionCount() const
    {
        return vDefinitionsM.Count();
    }

    String GetDefinition(U32 index) const
    {
        return vDefinitionsM[index];
    }

    U32 GetIncludeCount() const
    {
        return vIncludesM.Count();
    }

    String GetInclude(U32 index) const
    {
        return vIncludesM[index];
    }

    bool GetRtti() const
    {
        return !disableRttiM;
    }

    U32 GetHeaderCount() const
    {
        return vHeadersM.Count();
    }

    String GetHeader(U32 index) const
    {
        return vHeadersM[index];
    }

    U32 GetInputCount() const
    {
        return vInputsM.Count();
    }

    String GetInput(U32 index) const
    {
        return vInputsM[index];
    }

    String GetOutputFile() const
    {
        return outFileM;
    }

    String GetTempFile() const
    {
        return tmpFileM;
    }

    U32 GetCludeCount() const
    {
        return vCludesM.Count();
    }

    ref<Clude> GetClude(U32 index) const
    {
        return vCludesM[index];
    }
    
    bool ShouldInclude(const String &name) const;

private:

    Vector<String> vDefinitionsM;
    Vector<String> vIncludesM;
    RefVector<Clude> vCludesM;
    Vector<String> vHeadersM;
    bool disableRttiM;
    String outFileM;
    String tmpFileM;
    Vector<String> vInputsM;
};

}; // namespace Xrtti

#endif // CONFIGURATION_H
