/*****************************************************************************\
 *                                                                           *
 * CompiledContextSet.cpp                                                    *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
 * These are private data structure definitions shared between               *
 * xrttigen-generated code and the xrtti library.                            *
 *                                                                           *
 * THIS HEADER FILE IS NOT MEANT TO BE INSPECTED OR USED BY ANY END-USER OF  *
 * THE XRTTI LIBRARY.  IGNORE THE CONTENTS OF THIS FILE.                     *
 *                                                                           *
\*****************************************************************************/

#include <Essential/Hashtable.h>
#include <Essential/Log.h>
#include <Essential/String.h>
#include <Essential/Util.h>
#include <Essential/Vector.h>
#include "Xrtti/XrttiPrivate.h"


using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


static Hashtable<String, Context *> htNonAnonymousContextsByNameG;
static Hashtable<String, Structure *> htStructuresByTypeinfoG;
static Vector<Context *> vContextsG;


/* static */
U32 CompiledContextSet::GetContextCount()
{
    return vContextsG.Count();
}

    
/* static */
const Context *CompiledContextSet::GetContext(U32 index)
{
    return vContextsG[index];
}

    
/* static */
const Context *CompiledContextSet::LookupContext(const char *pFullName)
{
    Context *pRet = 0;

    if (!htNonAnonymousContextsByNameG.Get(pFullName, /* returns */ pRet)) {
        return 0;
    }

    return pRet;
}


/* static */
const Structure *
CompiledContextSet::LookupStructure(const std::type_info &typeinfo)
{
    Structure *pRet = 0;

    if (!htStructuresByTypeinfoG.Get(typeinfo.name(), /* returns */ pRet)) {
        return 0;
    }

    return pRet;
}


/* static */
void CompiledContextSet::RegisterContext(Context *pContext)
{
    Context *pExisting = 0;

    if ((pContext->GetType() == Context::Type_Namespace) ||
        !((Structure *) pContext)->IsAnonymous()) {
        if (htNonAnonymousContextsByNameG.Get(pContext->GetFullName(), 
                                              /* returns */ pExisting)) {
            switch (pContext->GetType()) {
            case Context::Type_Class:
                if (*((Class *) pContext) == *pExisting) {
                    return;
                }
                break;
            case Context::Type_Namespace:
                if (*((Namespace *) pContext) == *pExisting) {
                    return;
                }
                break;
            case Context::Type_Struct:
                if (*((Struct *) pContext) == *pExisting) {
                    return;
                }
                break;
            default: // Context::Type_Union
                if (*((Union *) pContext) == *pExisting) {
                    return;
                }
                break;
            }
            // This is bad - mismatch
            Log::ErrorF("Mismatched Contexts for %s\n", pContext->GetFullName());
            Util::Terminate();
        }
        
        htNonAnonymousContextsByNameG.Put(pContext->GetFullName(), pContext);
    }
        
    if (pContext->GetType() != Context::Type_Namespace) {
        Structure *pStructure = (Structure *) pContext;
        const std::type_info *pTypeInfo = pStructure->GetTypeInfo();
        if (pTypeInfo) {
            htStructuresByTypeinfoG.Put(pTypeInfo->name(), pStructure);
        }
    }

    vContextsG.Append(pContext);
}

}; // namespace Xrtti
