/*****************************************************************************\
 *                                                                           *
 * ParsedArgument.cpp                                                         *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "private/Parsed.h"

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


ParsedArgument::ParsedArgument()
    : pDefaultM(0)
{
}


ParsedArgument::~ParsedArgument()
{
    Memory<char>::Free(pDefaultM);
}


bool ParsedArgument::Initialize(ParsedContextSet &set, Parser &parser,
                                Parser::Element *pElement, String &error)
{
    // Type *pTypeM;
    if (!(pTypeM = set.GetType
          (parser, pElement->GetAttributeValue("type"), error))) {
        return false;
    }
    
    // bool hasDefaultM;
    hasDefaultM = pElement->HasAttribute("default");
    
    // Value defaultM;
    if (hasDefaultM) {
        // Cop-out: pDefaultM is just a pointer to the string itself
        pDefaultM = String::DuplicateChars
            (pElement->GetAttributeValue("default"));
    }
    
    return true;
}


void ParsedArgument::MergeContents(ParsedContextSet &to,
                                   ParsedContextSet &from)
{
    pTypeM = to.MergeType(pTypeM, from);
}


void ParsedArgument::ReplaceContext(Context *pOld, Context *pNew)
{
    switch (pTypeM->GetBaseType()) {
    case Type::BaseType_Function:
        ((ParsedTypeFunction *) pTypeM)->ReplaceContext(pOld, pNew);
        break;
    case Type::BaseType_Structure:
        ((ParsedTypeStructure *) pTypeM)->ReplaceContext(pOld, pNew);
        break;
    default:
        break;
    }
}


}; // namespace Xrtti
