/*****************************************************************************\
 *                                                                           *
 * ParsedConstructorSignature.cpp                                            *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "private/Parsed.h"

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


ParsedConstructorSignature::ParsedConstructorSignature()
    : pArgumentsM(0)
{
}

ParsedConstructorSignature::~ParsedConstructorSignature()
{
    delete [] pArgumentsM;
}

bool ParsedConstructorSignature::Initialize(ParsedContextSet &set,
                                            Parser &parser,
                                            Parser::Element *pElement,
                                            String &error)
{
    if (!superM.Initialize(set, parser, pElement, error)) {
        return false;
    }

    U32 count = pElement->GetSubElementCount();

    // U32 argumentCountM;
    argumentCountM = 0;
    for (U32 i = 0; i < count; i++) {
        ref<Parser::Element> pSubElement = pElement->GetSubElement(i);
            
        if (pSubElement->GetElementName() == "Argument") {
            argumentCountM++;
        }
    }

    // Argument *pArgumentsM;
    // bool hasEllipsisM;
    hasEllipsisM = false;
    if (argumentCountM) {
        pArgumentsM = new ParsedArgument[argumentCountM];

        U32 argumentIndex = 0;

        for (U32 i = 0; i < count; i++) {
            ref<Parser::Element> pSubElement = pElement->GetSubElement(i);
            if (pSubElement->GetElementName() == "Argument") {
                if (!pArgumentsM[argumentIndex++].Initialize
                    (set, parser, pSubElement, error)) {
                    return false;
                }
            }
            else if (pSubElement->GetElementName() == "Ellipsis") {
                hasEllipsisM = true;
            }
        }
    }

    return true;
}


void ParsedConstructorSignature::MergeContents(ParsedContextSet &to,
                                               ParsedContextSet &from)
{
    superM.MergeContents(to, from);

    for (U32 i = 0; i < argumentCountM; i++) {
        pArgumentsM[i].MergeContents(to, from);
    }
}


void ParsedConstructorSignature::ReplaceContext(Context *pOld, Context *pNew)
{
    for (U32 i = 0; i < argumentCountM; i++) {
        pArgumentsM[i].ReplaceContext(pOld, pNew);
    }
}


}; // namespace Xrtti
