/*****************************************************************************\
 *                                                                           *
 * ParsedContext.cpp                                                         *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "private/Parsed.h"

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


ParsedContext::ParsedContext()
    : pFullNameM(0)
{
}

bool ParsedContext::Initialize(ParsedContextSet &set, Parser &parser, 
                               const Parser::Element *pElement, String &error)
{
    // pNameM
    pNameM = get_name(pElement, true);

    // pContextM
    if (String::Equal(pNameM, "::")) {
        pContextM = 0;
    }
    else {
        if (!(pContextM = set.GetContext
              (parser, pElement->GetAttributeValue("context"), error))) {
            error = String::Format
                ("Failed to initialize Context id %S: %S",
                 (const unichar *) pElement->GetAttributeValue("id"),
                 (const unichar *) error);
            return false;
        }
    }
    
    // pFullNameM
    if (pContextM && pContextM->GetContext()) {
        String fullName = pContextM->GetFullName();
        fullName += "::";
        fullName += pNameM;
        pFullNameM = String::DuplicateChars(fullName);
    }
    else {
        pFullNameM = String::DuplicateChars(String(pNameM));
    }

    return true;
}

ParsedContext::~ParsedContext()
{
    Memory<char>::Free(pNameM);
    Memory<char>::Free(pFullNameM);
}


void ParsedContext::MergeContents(ParsedContextSet &to, ParsedContextSet &from)
{
    // pContextM
    if (pContextM) {
        pContextM = to.MergeContext(pContextM, from);
    }
}


}; // namespace Xrtti
