/*****************************************************************************\
 *                                                                           *
 * ParsedDestructorSignature.cpp                                             *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "private/Parsed.h"

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


ParsedDestructorSignature::ParsedDestructorSignature()
    : throwCountM(0), pThrowsM(0)
{
}

ParsedDestructorSignature::~ParsedDestructorSignature()
{
    delete [] pThrowsM;
}

bool ParsedDestructorSignature::Initialize(ParsedContextSet &set,
                                           Parser &parser,
                                           Parser::Element *pElement,
                                           String &error)
{
    String throws = pElement->GetAttributeValue("throw");

    // U32 throwCountM;
    RefVector<RefString> vThrows;
    split((const unichar *) throws, vThrows);
    throwCountM = vThrows.Count();

    // Type **pThrowsM;
    if (throwCountM) {
        pThrowsM = new Type * [throwCountM];
        for (U32 i = 0; i < throwCountM; i++) {
            pThrowsM[i] = 0;
        }

        for (U32 i = 0; i < throwCountM; i++) {
            String _throw = String(vThrows[i]);
                
            // If it's a context, then a type has to be used instead.  The
            // thing is, gccxml may not generate a Type for this and
            // instead just point to a context.  In this case, we have to
            // try to find a type which

            // It's either a Type or a Context.  Try to get it as a type.
            Type *pType = set.GetType(parser, _throw, error);

            if (!pType) {
                // Not a type ... try to get it as a structure
                Context *pContext = set.GetContext(parser, _throw, error);
                
                if (pContext) {
                    // Must convert this context into a type
                    pType = set.GetType(pContext, error);

                    if (!pType) {
                        error = String::Format
                            ("No such type while initializing destructor "
                             "signature id %S: %S", (const unichar *) 
                             pElement->GetAttributeValue("id"),
                             (const unichar *) error);
                        return false;
                    }
                }
                else {
                    // Not a context either
                    error = String::Format
                        ("No such context while initializing destructor "
                         "signature id %S: %S", (const unichar *) 
                         pElement->GetAttributeValue("id"),
                         (const unichar *) error);
                    return false;
                }
            }

            pThrowsM[i] = pType;
        }
    }

    return true;
}


void ParsedDestructorSignature::MergeContents(ParsedContextSet &to,
                                              ParsedContextSet &from)
{
    for (U32 i = 0; i < throwCountM; i++) {
        pThrowsM[i] = to.MergeType(pThrowsM[i], from);
    }
}


}; // namespace Xrtti
