/*****************************************************************************\
 *                                                                           *
 * ParsedEnumeration.cpp                                                     *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "private/Parsed.h"

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


ParsedEnumeration::ParsedEnumeration()
    : pNameM(0), pValuesM(0)
{
}

ParsedEnumeration::~ParsedEnumeration()
{
    Memory<char>::Free(pNameM);

    delete [] pValuesM;
}

bool ParsedEnumeration::Initialize(ParsedContextSet &set, Parser &parser,
                                   Parser::Element *pElement, String &error)
{
    // AccessType accessTypeM;
    accessTypeM = get_access_type(pElement);

    // Context *pContextM;
    if (!(pContextM = set.GetContext
          (parser, pElement->GetAttributeValue("context"), error))) {
        return false;
    }

    // const char *pNameM;
    pNameM = String::DuplicateChars(pElement->GetAttributeValue("name"));

    // U32 valueCountM;
    valueCountM = 0;

    U32 count = pElement->GetSubElementCount();

    for (U32 i = 0; i < count; i++) {
        ref<Parser::Element> pSubElement = pElement->GetSubElement(i);
            
        if (pSubElement->GetElementName() == "EnumValue") {
            valueCountM++;
        }
    }

    // EnumerationValue *pValuesM;
    if (valueCountM) {
        pValuesM = new ParsedEnumerationValue[valueCountM];

        count = 0;

        for (U32 i = 0; i < valueCountM; i++) {
            ref<Parser::Element> pSubElement = pElement->GetSubElement(i);
            
            if (pSubElement->GetElementName() != "EnumValue") {
                continue;
            }

            String name = pSubElement->GetAttributeValue("name");
            if (name == "") {
                error = String::Format
                    ("Enumeration value number %lu from Enumeration id %S "
                     "has no name", i, (const unichar *)
                     pElement->GetAttributeValue("id"));
                return false;
            }

            String value = pSubElement->GetAttributeValue("init");
            if (value == "") {
                error = String::Format
                    ("Enumeration value number %lu from Enumeration id %S "
                     "has no value", i, (const unichar *)
                     pElement->GetAttributeValue("id"));
                return false;
            }
            
            pValuesM[count++].Initialize(set, name, (S32) value.ToS64());
        }
    }

    return true;
}


void ParsedEnumeration::MergeContents(ParsedContextSet &to,
                                      ParsedContextSet &from)
{
    pContextM = to.MergeContext(pContextM, from);
}


const EnumerationValue &ParsedEnumeration::GetValue(U32 index) const
{
    return pValuesM[index];
}


}; // namespace Xrtti
