/*****************************************************************************\
 *                                                                           *
 * ParsedStruct.cpp                                                          *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "private/Parsed.h"

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


ParsedStruct::ParsedStruct()
    : pMethodsM(0)
{
}

ParsedStruct::~ParsedStruct()
{
    delete [] pMethodsM;
}

bool ParsedStruct::Initialize(ParsedContextSet &set, Parser &parser,
                              const Parser::Element *pElement, String &error)
{
    if (!superM.Initialize(set, parser, pElement, error)) {
        return false;
    }

    // Get a pointer to this context - this is done because this context
    // actually may not be a ParsedStructure; we cannot use our "this" pointer
    // as a pointer to this context, because we may just be imbedded in
    // another object (ParsedStruct or ParsedUnion)
    Context *pThisContext = set.GetContext
        (parser, pElement->GetAttributeValue("id"), error);

    // Get the members elements
    RefVector<Parser::Element> vMemberElements;

    if (!get_member_elements(parser, pElement, vMemberElements, error)) {
        return false;
    }

    // U32 methodCountM;
    methodCountM = 0;
    U32 count = vMemberElements.Count();
    for (U32 i = 0; i < count; i++) {
        ref<Parser::Element> pMemberElement = vMemberElements[i];

        String memberElementName = pMemberElement->GetElementName();

        if ((memberElementName == "Method") ||
            (memberElementName == "OperatorMethod")) {
            methodCountM++;
        }
        // Else ignore it
    }

    // Method *pMethodsM;
    if (methodCountM) {
        pMethodsM = new ParsedMethod[methodCountM];

        // Go over all members, collecting methods
        U32 methodIndex = 0;

        for (U32 i = 0; i < count; i++) {
            ref<Parser::Element> pMemberElement = vMemberElements[i];
                
            String memberElementName = pMemberElement->GetElementName();
                
            // If this member is a Method, then initialize it
            bool isOperatorMethod;
            if (memberElementName == "Method") {
                isOperatorMethod = false;
            }
            else if (memberElementName == "OperatorMethod") {
                isOperatorMethod = true;
            }
            // Otherwise, ignore it, we don't care about it
            else {
                continue;
            }

            if (!pMethodsM[methodIndex++].Initialize
                (set, parser, pThisContext, pMemberElement,
                 isOperatorMethod, error)) {
                return false;
            }
        }
    }

    // bool isAbstractM
    isAbstractM = false;
    for (U32 i = 0; i < methodCountM; i++) {
        if (pMethodsM[i].IsPureVirtual()) {
            isAbstractM = true;
            break;
        }
    }

    return true;
}


void ParsedStruct::MergeContents(ParsedContextSet &to, ParsedContextSet &from)
{
    superM.MergeContents(to, from);

    for (U32 i = 0; i < methodCountM; i++) {
        pMethodsM[i].MergeContents(to, from);
    }
}


void ParsedStruct::ReplaceContext(Context *pOld, Context *pNew)
{
    superM.ReplaceContext(pOld, pNew);

    for (U32 i = 0; i < methodCountM; i++) {
        pMethodsM[i].ReplaceContext(pOld, pNew);
    }
}


const Method &ParsedStruct::GetMethod(U32 index) const
{
    return pMethodsM[index];
}


}; // namespace Xrtti
