/*****************************************************************************\
 *                                                                           *
 * GeneratorArgument.cpp                                                     *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorArgument::GeneratorArgument(Generator &generator, const Argument &arg)
    : argumentM(arg)
{
    pTypeM = generator.GetGeneratorType(&arg.GetType());
}


const char *GeneratorArgument::GetTypeConstyThing()
{
    if (argumentM.GetType().IsConst() && 
        !argumentM.GetType().GetArrayOrPointerCount()) {
        return "const ";
    }

    return "";
}


U32 GeneratorArgument::GetTypeNumber()
{
    return pTypeM->GetNumber();
}


bool GeneratorArgument::CanEmitType()
{
    return pTypeM->CanEmitTypedef();
}


void GeneratorArgument::EmitTypedefs(OutputStream &out)
{
    pTypeM->EmitTypedef(out);
}


void GeneratorArgument::EmitExtraDefinitions(OutputStream &out)
{
    if (argumentM.HasDefault()) {
        out.WriteFA("    static XrttiAccess::_%lu_type _%lu_default = %s;\n\n",
                    pTypeM->GetNumber(), this->GetNumber(), 
                    * (const char **) argumentM.GetDefault());
    }
}


void GeneratorArgument::EmitDefinitionArguments(OutputStream &out)
{
    out.WriteFA("        &_::_%lu,\n", pTypeM->GetNumber());

    Generator::EmitBooleanArgument(out, argumentM.HasDefault(), true);

    if (argumentM.HasDefault()) {
        out.WriteFA("        &_%lu_default", this->GetNumber());
    }
    else {
        out.WriteFA("        0");
    }
}


}; // namespace Xrtti
