/*****************************************************************************\
 *                                                                           *
 * GeneratorBase.cpp                                                         *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorBase::GeneratorBase(Generator &generator, const Base &base,
                             GeneratorStructure *pSubstructure)
    : baseM(base), pSubstructureM(pSubstructure)
{
    pStructureM = (GeneratorStructure *) 
        generator.GetGeneratorContext(&(base.GetStructure()));
}


void GeneratorBase::EmitXrttiAccess(OutputStream &out)
{
    if (!this->CanEmitCast()) {
        return;
    }

    out.WriteFA("    static void *_%lu_cast(void *pSubclass)\n    {\n",
                this->GetNumber());
    
    out.WriteFA("        return (%S *) (%S *) pSubclass;\n",
                (const unichar *) Generator::GetTypeName
                (&(baseM.GetStructure())),
                (const unichar *) pSubstructureM->GetTypeName());

    out.WriteA("    };\n\n");
}


void GeneratorBase::EmitDefinitionArguments(OutputStream &out)
{
    Generator::EmitAccessTypeArgument(out, baseM.GetAccessType(), true);

    Generator::EmitBooleanArgument(out, baseM.IsVirtual(), true);

    out.WriteFA("        &_::_%lu,\n", pStructureM->GetNumber());

    if (this->CanEmitCast()) {
        out.WriteFA("        &XrttiAccess::_%lu_cast", this->GetNumber());
    }
    else {
        Generator::EmitU32Argument(out, 0, false);
    }
}


bool GeneratorBase::CanEmitCast()
{
    return (Generator::IsAccessible(&(baseM.GetStructure())) &&
            pSubstructureM->IsAccessible());
}


}; // namespace Xrtti
