/*****************************************************************************\
 *                                                                           *
 * GeneratorConstructor.cpp                                                  *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


bool GeneratorConstructor::IsAccessible()
{
    return Generator::IsAccessible(&constructorM);
}


bool GeneratorConstructor::CanInvoke()
{
    // Only non-anonymous, accessible context constructors can be emitted
    const Structure &structure = 
        (const Structure &) (constructorM.GetContext());

    if (structure.IsAnonymous() || 
        (((structure.GetType() == Context::Type_Class) ||
          (structure.GetType() == Context::Type_Struct)) &&
         ((const Struct &) structure).IsAbstract())) {
        return false;
    }

    // Check the signature
    return constructorSignatureM.CanEmitArguments();
}


void GeneratorConstructor::EmitTypedefs(OutputStream &out)
{
    if (!Generator::IsAccessible(&constructorM) || !this->CanInvoke()) {
        return;
    }

    constructorSignatureM.EmitTypedefs(out);
}


void GeneratorConstructor::EmitXrttiAccess(OutputStream &out)
{
    if (!Generator::IsAccessible(&constructorM) || !this->CanInvoke()) {
        return;
    }

    out.WriteFA("    static void *_%lu_invoke(void **pArgs)\n    {\n",
                this->GetNumber());

    out.WriteFA("        return new %S(",
                (const unichar *) Generator::GetTypeName
                (&(constructorM.GetContext())));

    constructorSignatureM.EmitInvokeArguments(out);
    
    out.WriteA(");\n    }\n\n");
}


void GeneratorConstructor::EmitExtraDeclarations(OutputStream &out)
{
    constructorSignatureM.EmitDeclaration(out);
}


void GeneratorConstructor::EmitExtraDefinitions(OutputStream &out)
{
    constructorSignatureM.EmitDefinition(out);

    U32 count = constructorSignatureM.GetArgumentCount();
    if (count) {
        out.WriteFA("    static char *_%lu_names[] =\n    {\n", 
                    this->GetNumber());
        for (U32 i = 0; i < count; i++) {
            out.WriteFA("        \"%s\"%s\n", constructorM.GetArgumentName(i),
                        (i < (count - 1)) ? "," : "");
        }
        out.WriteA("    };\n\n");
    }
}


void GeneratorConstructor::EmitDefinitionArguments(OutputStream &out)
{
    this->GeneratorMember::EmitDefinitionArguments(out);
    out.WriteA(",\n");

    out.WriteFA("        &_::_%lu,\n", constructorSignatureM.GetNumber());

    if (constructorSignatureM.GetArgumentCount()) {
        out.WriteFA("        _%lu_names,\n", this->GetNumber());
    }
    else {
        out.WriteFA("        0,\n");
    }

    if (Generator::IsAccessible(&constructorM) && this->CanInvoke()) {
        out.WriteFA("        &XrttiAccess::_%lu_invoke", this->GetNumber());
    }
    else {
        Generator::EmitU32Argument(out, 0, false);
    }
}


}; // namespace Xrtti
