/*****************************************************************************\
 *                                                                           *
 * GeneratorConstructorSignature.cpp                                         *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorConstructorSignature::
GeneratorConstructorSignature(Generator &generator,
                              const ConstructorSignature &sig)
    : GeneratorDestructorSignature(generator, 
                                   (const DestructorSignature &) sig),
      constructorSignatureM(sig)
{
    U32 count = sig.GetArgumentCount();
    if (count) {
        pArgumentsM = new GeneratorArgument *[count];
        for (U32 i = 0; i < count; i++) {
            pArgumentsM[i] = new GeneratorArgument
                (generator, sig.GetArgument(i));
        }
    }
    else {
        pArgumentsM = 0;
    }
}


GeneratorConstructorSignature::~GeneratorConstructorSignature()
{
    U32 count = constructorSignatureM.GetArgumentCount();
    for (U32 i = 0; i < count; i++) {
        delete pArgumentsM[i];
    }

    delete [] pArgumentsM;
}


bool GeneratorConstructorSignature::CanEmitArguments()
{
    U32 count = constructorSignatureM.GetArgumentCount();

    for (U32 i = 0; i < count; i++) {
        if (!pArgumentsM[i]->CanEmitType()) {
            return false;
        }
    }

    return true;
}


void GeneratorConstructorSignature::EmitInvokeArguments(OutputStream &out)
{
    U32 count = constructorSignatureM.GetArgumentCount();

    for (U32 i = 0; i < count; i++) {
        out.WriteFA("* (%s_%lu_type *) pArgs[%lu]%s",
                    pArgumentsM[i]->GetTypeConstyThing(),
                    pArgumentsM[i]->GetTypeNumber(), i,
                    (i < (count - 1)) ? ", " : "");
    }
}


void GeneratorConstructorSignature::EmitTypedefs(OutputStream &out)
{
    U32 count = constructorSignatureM.GetArgumentCount();
    for (U32 i = 0; i < count; i++) {
        pArgumentsM[i]->EmitTypedefs(out);
    }
}


void GeneratorConstructorSignature::EmitArgumentTypeList(OutputStream &out)
{
    U32 count = constructorSignatureM.GetArgumentCount();
    for (U32 i = 0; i < count; i++) {
        out.WriteFA("_%lu_type%s", pArgumentsM[i]->GetTypeNumber(),
                    (i < (count - 1)) ? ", " : "");
    }
}


void GeneratorConstructorSignature::EmitExtraDeclarations(OutputStream &out)
{
    this->GeneratorDestructorSignature::EmitExtraDeclarations(out);

    U32 count = constructorSignatureM.GetArgumentCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pArgumentsM[i]->EmitDeclaration(out);
        }
    }
}


void GeneratorConstructorSignature::EmitExtraDefinitions(OutputStream &out)
{
    this->GeneratorDestructorSignature::EmitExtraDefinitions(out);

    U32 count = constructorSignatureM.GetArgumentCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pArgumentsM[i]->EmitDefinition(out);
        }
        out.WriteFA("    static Xrtti::Argument *_%lu_arguments[] ="
                    "\n    {\n", this->GetNumber());
        for (U32 i = 0; i < count; i++) {
            out.WriteFA("        &_::_%lu%s\n", pArgumentsM[i]->GetNumber(),
                        (i < (count - 1)) ? "," : "");
        }
        out.WriteA("    };\n\n");
    }
}


void GeneratorConstructorSignature::EmitDefinitionArguments(OutputStream &out)
{
    this->GeneratorDestructorSignature::EmitDefinitionArguments(out);
    out.WriteA(",\n");

    Generator::EmitArrayArgument(out, constructorSignatureM.GetArgumentCount(),
                                 "arguments", this->GetNumber(), true);

    Generator::EmitBooleanArgument
        (out, constructorSignatureM.HasEllipsis(), false);
}


}; // namespace Xrtti
