/*****************************************************************************\
 *                                                                           *
 * GeneratorDestructor.cpp                                                   *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


bool GeneratorDestructor::IsAccessible()
{
    return Generator::IsAccessible(&destructorM);
}


bool GeneratorDestructor::CanInvoke()
{
    // Only non-anonymous, accessible context constructors can be emitted
    const Structure &structure = 
        (const Structure &) (destructorM.GetContext());

    if (structure.IsAnonymous() || 
        (((structure.GetType() == Context::Type_Class) ||
          (structure.GetType() == Context::Type_Struct)) &&
         ((const Struct &) structure).IsAbstract()) ||
        !this->IsAccessible()) {
        return false;
    }

    return true;
}


void GeneratorDestructor::EmitXrttiAccess(OutputStream &out)
{
    if (!Generator::IsAccessible(&destructorM) || !this->CanInvoke()) {
        return;
    }

    out.WriteFA("    static void _%lu_invoke(void *pThis)\n    {\n",
                this->GetNumber());

    out.WriteFA("        delete (%S *) pThis;\n", 
                (const unichar *) Generator::GetTypeName
                (&(destructorM.GetContext())));

    out.WriteA("    }\n\n");
}


void GeneratorDestructor::EmitExtraDeclarations(OutputStream &out)
{
    destructorSignatureM.EmitDeclaration(out);
}


void GeneratorDestructor::EmitExtraDefinitions(OutputStream &out)
{
    destructorSignatureM.EmitDefinition(out);
}


void GeneratorDestructor::EmitDefinitionArguments(OutputStream &out)
{
    this->GeneratorMember::EmitDefinitionArguments(out);
    out.WriteA(",\n");

    Generator::EmitBooleanArgument(out, destructorM.IsVirtual(), true);

    Generator::EmitBooleanArgument(out, destructorM.IsPureVirtual(), true);

    out.WriteFA("        &_::_%lu,\n", destructorSignatureM.GetNumber());

    if (Generator::IsAccessible(&destructorM) && this->CanInvoke()) {
        out.WriteFA("        &XrttiAccess::_%lu_invoke", this->GetNumber());
    }
    else {
        Generator::EmitU32Argument(out, 0, false);
    }
}


}; // namespace Xrtti
