/*****************************************************************************\
 *                                                                           *
 * GeneratorDestructorSignature.cpp                                          *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorDestructorSignature::
GeneratorDestructorSignature(Generator &generator,
                             const DestructorSignature &destructorSignature)
    : destructorSignatureM(destructorSignature)
{
    U32 count = destructorSignature.GetThrowCount();
    if (count) {
        pThrowsM = new GeneratorType *[count];
        for (U32 i = 0; i < count; i++) {
            pThrowsM[i] = generator.GetGeneratorType
                (&destructorSignature.GetThrow(i));
        }
    }
    else {
        pThrowsM = 0;
    }
}


GeneratorDestructorSignature::~GeneratorDestructorSignature()
{
    delete [] pThrowsM;
}


void GeneratorDestructorSignature::EmitExtraDefinitions(OutputStream &out)
{
    U32 count = destructorSignatureM.GetThrowCount();
    if (count) {
        out.WriteFA("    static Xrtti::Type *_%lu_throws[] =\n    {\n", 
                    this->GetNumber());
        for (U32 i = 0; i < count; i++) {
            out.WriteFA("        &_::_%lu%s\n", pThrowsM[i]->GetNumber(),
                        (i < (count - 1)) ? "," : "");
        }
        
        out.WriteFA("    };\n\n");
    }
}


void GeneratorDestructorSignature::EmitDefinitionArguments(OutputStream &out)
{
    Generator::EmitArrayArgument(out, destructorSignatureM.GetThrowCount(),
                                 "throws", this->GetNumber(), false);
}


}; // namespace Xrtti
