/*****************************************************************************\
 *                                                                           *
 * GeneratorEnumeration.cpp                                                  *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorEnumeration::GeneratorEnumeration(Generator &generator, 
                                           const Enumeration &enumeration)
    : enumerationM(enumeration)
{
    generator.RegisterGeneratorEnumeration(&enumeration, this);

    pContextM = generator.GetGeneratorContext(&(enumeration.GetContext()));

    U32 count = enumerationM.GetValueCount();
    if (count) {
        pValuesM = new GeneratorEnumerationValue *[count];
        for (U32 i = 0; i < count; i++) {
            pValuesM[i] = new GeneratorEnumerationValue
                (enumerationM.GetValue(i));
        }
    }
    else {
        pValuesM = 0;
    }
}


GeneratorEnumeration::~GeneratorEnumeration()
{
    U32 count = enumerationM.GetValueCount();
    for (U32 i = 0; i < count; i++) {
        delete pValuesM[i];
    }
    delete [] pValuesM;
}


void GeneratorEnumeration::EmitExtraDeclarations(OutputStream &out)
{
    U32 count = enumerationM.GetValueCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pValuesM[i]->EmitDeclaration(out);
        }
    }
}


void GeneratorEnumeration::EmitExtraDefinitions(OutputStream &out)
{
    U32 count = enumerationM.GetValueCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pValuesM[i]->EmitDefinition(out);
        }
        out.WriteFA("    static Xrtti::EnumerationValue *_%lu_values[] ="
                    "\n    {\n", this->GetNumber());
        for (U32 i = 0; i < count; i++) {
            out.WriteFA("        &_::_%lu%s\n", pValuesM[i]->GetNumber(),
                        (i < (count - 1)) ? "," : "");
        }
        out.WriteA("    };\n\n");
    }
}


void GeneratorEnumeration::EmitDefinitionArguments(OutputStream &out)
{
    Generator::EmitAccessTypeArgument(out, enumerationM.GetAccessType(), true);

    out.WriteFA("        &_::_%lu,\n", pContextM->GetNumber());

    Generator::EmitStringArgument(out, enumerationM.GetName(), true);

    Generator::EmitArrayArgument
        (out, enumerationM.GetValueCount(), "values", this->GetNumber(), false);
}


}; // namespace Xrtti
