/*****************************************************************************\
 *                                                                           *
 * Generator.cpp                                                             *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorStruct::GeneratorStruct(Generator &generator, const Struct &structRef)
    : GeneratorStructure(generator, (const Structure &) structRef),
      structM(structRef)
{
    U32 count = structRef.GetMethodCount();
    if (count) {
        pMethodsM = new GeneratorMethod *[count];
        for (U32 i = 0; i < count; i++) {
            pMethodsM[i] = new GeneratorMethod
                (generator, structRef.GetMethod(i));
        }
    }
    else {
        pMethodsM = 0;
    }
}


GeneratorStruct::~GeneratorStruct()
{
    U32 count = structM.GetMethodCount();
    for (U32 i = 0; i < count; i++) {
        delete pMethodsM[i];
    }
    delete [] pMethodsM;
}


void GeneratorStruct::EmitXrttiAccess(OutputStream &out)
{
    this->GeneratorStructure::EmitXrttiAccess(out);

    U32 count = structM.GetMethodCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pMethodsM[i]->EmitXrttiAccess(out);
        }
    }
}


void GeneratorStruct::EmitNeededTypedefs(OutputStream &out)
{
    this->GeneratorStructure::EmitNeededTypedefs(out);

    U32 count = structM.GetMethodCount();
    for (U32 i = 0; i < count; i++) {
        pMethodsM[i]->EmitTypedefs(out);
    }
}


void GeneratorStruct::EmitExtraDeclarations(OutputStream &out)
{
    this->GeneratorStructure::EmitExtraDeclarations(out);

    U32 count = structM.GetMethodCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pMethodsM[i]->EmitDeclaration(out);
        }
    }
}


void GeneratorStruct::EmitExtraDefinitions(OutputStream &out)
{
    this->GeneratorStructure::EmitExtraDefinitions(out);

    U32 count = structM.GetMethodCount();
    if (count) {
        for (U32 i = 0; i < count; i++) {
            pMethodsM[i]->EmitDefinition(out);
        }

        out.WriteFA("    static Xrtti::Method *_%lu_methods[] ="
                    "\n    {\n", this->GetNumber());
        for (U32 i = 0; i < count; i++) {
            out.WriteFA("        &_::_%lu%s\n", pMethodsM[i]->GetNumber(),
                        (i < (count - 1)) ? "," : "");
        }

        out.WriteFA("    };\n\n");
    }
}


void GeneratorStruct::EmitDefinitionArguments(OutputStream &out)
{
    this->GeneratorStructure::EmitDefinitionArguments(out);
    out.WriteA(",\n");

    // isAbstract
    Generator::EmitBooleanArgument(out, structM.IsAbstract(), true);

    // methodCount
    // pMethods
    Generator::EmitArrayArgument
        (out, structM.GetMethodCount(), "methods", this->GetNumber(), false);
}



}; // namespace Xrtti
