/*****************************************************************************\
 *                                                                           *
 * GeneratorTypeFunction.cpp                                              *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/


#include "private/Generator.h"

using namespace Essential;

namespace Xrtti {

#if 0 // This fixes indentation in emacs
}
#endif


GeneratorTypeFunction::
GeneratorTypeFunction(Generator &generator, const TypeFunction &typeFunction)
    : GeneratorType(generator, (const Type &) typeFunction),
      typeFunctionM(typeFunction)
{
    pMethodSignatureM = new GeneratorMethodSignature
        (generator, typeFunction.GetSignature());
}


bool GeneratorTypeFunction::CanEmitTypedef()
{
    return pMethodSignatureM->CanEmitArguments();
}


void GeneratorTypeFunction::EmitPrerequisiteTypedefs(OutputStream &out)
{
    pMethodSignatureM->EmitTypedefs(out);
}


void GeneratorTypeFunction::EmitTypedefFunctionArguments(OutputStream &out)
{
    out.WriteA("(");
    pMethodSignatureM->EmitArgumentTypeList(out);
    out.WriteA(")");
}


String GeneratorTypeFunction::GetTypedefTypeName()
{
    return String::Format("_%lu_type", 
                          pMethodSignatureM->GetReturnTypeNumber());
}


void GeneratorTypeFunction::EmitExtraDeclarations(OutputStream &out)
{
    this->GeneratorType::EmitExtraDeclarations(out);

    pMethodSignatureM->EmitDeclaration(out);
}


void GeneratorTypeFunction::EmitExtraDefinitions(OutputStream &out)
{
    this->GeneratorType::EmitExtraDefinitions(out);

    pMethodSignatureM->EmitDefinition(out);
}


void GeneratorTypeFunction::EmitDefinitionArguments(OutputStream &out)
{
    this->GeneratorType::EmitDefinitionArguments(out);
    out.WriteA(",\n");
    
    out.WriteFA("        &_::_%lu", pMethodSignatureM->GetNumber());
}


}; // namespace Xrtti
