/*****************************************************************************\
 *                                                                           *
 * xrttigen.cpp                                                              *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
\*****************************************************************************/

#include "Essential/Log.h"
#include "Xrtti/Xrtti.h"
#include "private/Configuration.h"
#include "private/Generator.h"

using namespace Xrtti;
#define U32 Xrtti::U32


int main(int argc, char **argv)
{
    // Read the configuration from the arguments
    Configuration config(argc, argv);

    // Get the include list and definition list
    U32 includeCount = config.GetIncludeCount();
    char **pIncludes = new char * [includeCount];
    for (U32 i = 0; i < includeCount; i++) {
        pIncludes[i] = String::DuplicateChars(config.GetInclude(i));
    }

    U32 definitionCount = config.GetDefinitionCount();
    char **pDefinitions = new char * [definitionCount];
    for (U32 i = 0; i < definitionCount; i++) {
        pDefinitions[i] = String::DuplicateChars(config.GetDefinition(i));
    }

    // Create a ContextSet to read in all of the contexts from the
    // header file(s)
    ContextSet *pContextSet = CreateContextSet();

    bool success = true;
    U32 inputCount = config.GetInputCount();
    for (U32 i = 0; i < inputCount; i++) {
        char *pInput = String::DuplicateChars(config.GetInput(i));
        char *pTmp = String::DuplicateChars(config.GetTempFile());
        if (!pContextSet->AddHeader(pInput, includeCount, 
                                    (const char **) pIncludes,
                                    definitionCount, 
                                    (const char **) pDefinitions, pTmp)) {
            Log::ErrorF("Failed to process header file: %s\n", pInput);
            Log::Error(pContextSet->GetLastError());
            Log::Error("\n");
            Memory<char>::Free(pInput);
            success = false;
            break;
        }
        Memory<char>::Free(pInput);
    }

    for (U32 i = 0; i < definitionCount; i++) {
        Memory<char>::Free(pDefinitions[i]);
    }
    delete [] pDefinitions;

    for (U32 i = 0; i < includeCount; i++) {
        Memory<char>::Free(pIncludes[i]);
    }
    delete [] pIncludes;

    if (!success) {
        delete pContextSet;
        return -1;
    }

    {
        Generator generator(config, *pContextSet);

        if (!generator.Generate()) {
            delete pContextSet;
            return -1;
        }
    }

    delete pContextSet;

    return 0;
}
